#include "moo/dialogs.h"
#include "moo/dialogs-p.h"
#include "moo/utils.h"
#include "moo/app.h"
#include "moo/prefs.h"
#include "moo/docwindow.h"
#include "moo/document.h"
#include "moo/macros.h"
#include <QtGui>

using namespace moo;

static QList<QUrl> openFileNames_KDE(DocumentWindow *window, const QList<dlg::FileFilterInfo> &filters)
{
    QStringList args;
    args << "kdialog" << "--multiple";

    QWidget *parent = window;
    if (!parent)
        parent = QApplication::activeWindow();

    if (parent)
        // cast WinID to qint64 to avoid compilation errors on windows
        args << "--embed" << QString::number((qint64) parent->effectiveWinId());

    args << "--getopenurl";

    QString startdir = stateValue("dialogs/open-file").toString();
    args << startdir;

    if (!filters.isEmpty())
    {
        QStringList filterString;
        foreach (const dlg::FileFilterInfo &ffi, filters)
            filterString << QString("%1|%2 (%1)").arg(ffi.filter, ffi.name);
        args << filterString.join("\n");
    }

    QByteArray out, err;
    int exitCode = impl::MyProcess::run(args, out, err);

    mDebug("devel") << "out: " << out;
    mDebug("devel") << "err: " << err;

    if (exitCode != 0)
        return QList<QUrl>();

    QList<QByteArray> strings = out.simplified().split(' ');

    QList<QUrl> urls;
    foreach (const QByteArray &a, strings)
        urls << QUrl::fromEncoded(a);

    m_return_val_if_fail(!urls.isEmpty(), urls);

    {
        QString url = urls[0].toString();
        int lastSlash = url.lastIndexOf('/');
        if (lastSlash > 0 && lastSlash + 1 != url.length())
            setStateValue("dialogs/open-file", url.mid(0, lastSlash));
        else
            setStateValue("dialogs/open-file", url);
    }

    mDebug("devel") << "urls: " << urls;
    return urls;
}

static QUrl saveFileName_KDE(DocumentWindow *window,
                             const QList<dlg::FileExtensionInfo> &extensions,
                             dlg::FileExtensionInfo *ext)
{
    QStringList args;
    args << "kdialog";

    QWidget *parent = window;
    if (!parent)
        parent = QApplication::activeWindow();

    if (parent)
        // cast WinID to qint64 to avoid compilation errors on windows
        args << "--embed" << QString::number((qint64) parent->effectiveWinId());

    args << "--getsaveurl";

    QString startdir = stateValue("dialogs/save-file").toString();
    args << startdir;

    if (!extensions.isEmpty())
    {
        *ext = extensions[0];

        QStringList list;
        foreach (const dlg::FileExtensionInfo &fe, extensions)
            list << QString("*%1|%2 (*%1)").arg(fe.extension, fe.name);
        args << list.join("\n");
    }

    QByteArray out, err;
    int exitCode = impl::MyProcess::run(args, out, err);

    mDebug("devel") << "out: " << out;
    mDebug("devel") << "err: " << err;

    if (exitCode != 0)
        return QUrl();

    QList<QByteArray> strings = out.simplified().split(' ');
    m_return_val_if_fail(!strings.isEmpty(), QUrl());

    QUrl url = QUrl::fromEncoded(strings[0]);
    QString urlString = url.toString();

    foreach (const dlg::FileExtensionInfo &fe, extensions)
    {
        if (urlString.endsWith(fe.extension))
        {
            *ext = fe;
            break;
        }
    }

    {
        int lastSlash = urlString.lastIndexOf('/');
        if (lastSlash > 0 && lastSlash + 1 != urlString.length())
            setStateValue("dialogs/save-file", urlString.mid(0, lastSlash));
        else
            setStateValue("dialogs/save-file", urlString);
    }

    mDebug("devel") << "url: " << url;
    return url;
}

static QString filterString_Qt(const QList<dlg::FileFilterInfo> &filters)
{
    QStringList list;
    foreach (const dlg::FileFilterInfo &ffi, filters)
        list << QString("%2 (%1)").arg(ffi.filter, ffi.name);
    return list.join(";;");
}

static QList<QUrl> openFileNames_Qt(DocumentWindow *window, const QList<dlg::FileFilterInfo> &filters)
{
    QString start;

#if !defined Q_OS_MAC && !defined Q_OS_WIN32
    start = stateValue("dialogs/open-file").toString();
#endif

    QWidget *parent = window;
    QFileDialog::Options options = QFileDialog::DontResolveSymlinks;

#ifdef Q_OS_MAC
    if (window && !window->document()->isEmpty())
        options |= QFileDialog::DontUseSheet;
#endif

    QStringList filenames;
    filenames = QFileDialog::getOpenFileNames(parent,
                                              DocumentWindow::tr("Open File"),
                                              start,
                                              filterString_Qt(filters),
                                              0,
                                              options);

#if !defined Q_OS_MAC && !defined Q_OS_WIN32
    if (!filenames.isEmpty())
        setStateValue("dialogs/open-file", QFileInfo(filenames[0]).path());
#endif

    QList<QUrl> urls;
    foreach (const QString &fn, filenames)
        urls << QUrl::fromLocalFile(fn);
    return urls;
}

QList<QUrl> dlg::openFileNames(DocumentWindow *window, const QList<FileFilterInfo> &filters)
{
    if (0)
        return openFileNames_KDE(window, filters);
    else
        return openFileNames_Qt(window, filters);
}


static QString getNameFromWindow(DocumentWindow *window)
{
    QString name;

    if (window)
    {
        name = window->document()->url().toString();
        if (name.isEmpty())
            name = window->document()->displayName();
    }

    return name;
}

static QString filterString_Qt(const QList<dlg::FileExtensionInfo> &extensions)
{
    QStringList list;
    foreach (const dlg::FileExtensionInfo &fe, extensions)
        list << QString("%1 (*%2)").arg(fe.name, fe.extension);
    return list.join(";;");
}

static dlg::FileExtensionInfo extensionFromQtFilter(const QString &filter,
                                                    const QList<dlg::FileExtensionInfo> &extensions)
{
    foreach (const dlg::FileExtensionInfo &fe, extensions)
        if (QString("%1 (*%2)").arg(fe.name, fe.extension) == filter)
            return fe;
    return dlg::FileExtensionInfo();
}

static QUrl saveFileName_Native(DocumentWindow *window, const QList<dlg::FileExtensionInfo> &extensions, dlg::FileExtensionInfo *ext)
{
    QString filter;
    QString filename = QFileDialog::getSaveFileName(window, DocumentWindow::tr("Save As"),
                                                    getNameFromWindow(window),
                                                    filterString_Qt(extensions),
                                                    &filter,
                                                    QFileDialog::DontResolveSymlinks);
    if (!filename.isEmpty())
        *ext = extensionFromQtFilter(filter, extensions);
    return filename.isEmpty() ? QUrl() : QUrl::fromLocalFile(filename);
}

#define PREFS_SAVE_DIALOG_STATE "ui/save-dialog-state"

static QUrl saveFileName_Qt(DocumentWindow *window, const QList<dlg::FileExtensionInfo> &extensions, dlg::FileExtensionInfo *ext)
{
    QPointer<DocumentWindow> window_ptr = window;

    QFileDialog dlg(window, DocumentWindow::tr("Save As"),
                    getNameFromWindow(window),
                    filterString_Qt(extensions));

    dlg.setAcceptMode(QFileDialog::AcceptSave);
    dlg.setFileMode(QFileDialog::AnyFile);
    dlg.setResolveSymlinks(false);
    dlg.setConfirmOverwrite(false);

    QByteArray state = stateValue(PREFS_SAVE_DIALOG_STATE).toByteArray();
    if (!state.isEmpty())
        dlg.restoreState(state);

    QString filename;
    dlg::FileExtensionInfo selectedExtension;

    while (dlg.exec())
    {
        filename = dlg.selectedFiles().value(0);
        selectedExtension = extensionFromQtFilter(dlg.selectedFilter(), extensions);

        if (QFileInfo(filename).suffix().isEmpty() && !selectedExtension.extension.isEmpty())
            filename += selectedExtension.extension;

        if (!QFileInfo(filename).exists())
            break;

        QString basename = QFileInfo(filename).fileName();
        QMessageBox::StandardButton btn =
            QMessageBox::warning(&dlg, "",
                                 DocumentWindow::tr("A file named \"%1\" already exists in this location. "
                                                    "Do you want to replace it with the one you are saving?").arg(basename),
                                 QMessageBox::Ok | QMessageBox::Cancel, QMessageBox::Cancel);
        if (btn == QMessageBox::Ok)
            break;

        filename = QString();
    }

    if (!filename.isEmpty())
        *ext = selectedExtension;

    state = dlg.saveState();
    setStateValue(PREFS_SAVE_DIALOG_STATE, state);

    if (window_ptr)
        window_ptr->activateWindow();

    return filename.isEmpty() ? QUrl() : QUrl::fromLocalFile(filename);
}

#if 1 && (defined(Q_OS_MAC) || defined(Q_OS_WIN32))
#define USE_NATIVE_DIALOG true
#define USE_KDE_DIALOG false
#else // unix
#define USE_NATIVE_DIALOG false
#define USE_KDE_DIALOG false
#endif // unix

QUrl dlg::saveFileName(DocumentWindow *window, const QList<FileExtensionInfo> &extensions, FileExtensionInfo *ext)
{
    if (USE_NATIVE_DIALOG)
        return saveFileName_Native(window, extensions, ext);
    else if (USE_KDE_DIALOG)
        return saveFileName_KDE(window, extensions, ext);
    else
        return saveFileName_Qt(window, extensions, ext);
}


QString dlg::exeFileName(QWidget *parent, const QString &title, const QString &start)
{
    return QFileDialog::getOpenFileName(parent, title, start, QString(), 0,
                                        // FIXME Qt 4.4 ignores this flag on Mac
                                        QFileDialog::DontResolveSymlinks);
}

QString dlg::dirName(QWidget *parent, const QString &title, const QString &start)
{
    return QFileDialog::getExistingDirectory(parent, title, start,
                                             // FIXME Qt 4.4 ignores this flag on Mac
                                             QFileDialog::DontResolveSymlinks | QFileDialog::ShowDirsOnly);
}


NiceDialog::NiceDialog(QWidget *parent) :
    QDialog(parent)
{
    setAttribute(Qt::WA_DeleteOnClose);
    setAttribute(Qt::WA_QuitOnClose, false);
    new QShortcut(QKeySequence::Close, this, SLOT(close()));
    new QShortcut(QKeySequence("Ctrl+Q"), this, SLOT(quit()));
}

NiceDialog::~NiceDialog()
{
}

void NiceDialog::quit()
{
    mooApp->maybeQuit();
}


void moo::dlg::openError(QWidget *window, const QString &filename, const QString &error)
{
    QMessageBox::critical(window, "", AppWindow::tr("Could not open file %1\n%2").arg(filename).arg(error));
}

void moo::dlg::saveError(QWidget *window, const QString &filename, const QString &error)
{
    QMessageBox::critical(window, "", AppWindow::tr("Could not save file %1\n%2").arg(filename).arg(error));
}


QMessageBox *moo::dlg::saveChanges(QWidget *window)
{
    QMessageBox *msgBox = new QMessageBox(window);
    msgBox->setAttribute(Qt::WA_DeleteOnClose);
    msgBox->setStandardButtons(QMessageBox::Discard | QMessageBox::Cancel | QMessageBox::Save);
    msgBox->setDefaultButton(QMessageBox::Save);
    msgBox->setText(AppWindow::tr("Do you want to save changes to this document before closing?"));
    msgBox->setInformativeText(AppWindow::tr("If you don't save, your changes will be lost."));
    msgBox->setIcon(QMessageBox::Question);
    msgBox->setWindowModality(Qt::WindowModal);
    return msgBox;
}

static QString makePrimaryText(const QString &primaryText)
{
    QString text = primaryText;
    const QString tmpl("This document could not be saved because");
    for (int i = 0; i < tmpl.length() - primaryText.length(); ++i)
        text.append("  ");
    return text;
}

QMessageBox::StandardButton
moo::dlg::information(QWidget *parent,
                       const QString &title,
                       const QString &text,
                       const QString &secondaryText,
                       QMessageBox::StandardButtons buttons,
                       QMessageBox::StandardButton defaultButton)
{
    QMessageBox box(QMessageBox::Information, title,
                    makePrimaryText(text), buttons, parent);
    box.setDefaultButton(defaultButton);
    box.setInformativeText(secondaryText);
    return QMessageBox::StandardButton(box.exec());
}
